@@ -1,4 +1,4 @@
-library("lava")
+#library("lava")
 suppressPackageStartupMessages(library("testthat"))
 test_check("lava")
 
